--******************************************************************************************************
--[[
	SimpleRangeIndicator
	Based on SimpleRange by:

	Autor: Bo
	E-Mail: webmaster@myrtana.de

	Acefied: Elsia, based on InCombatIndicator, to remove need for move addon.

	Vielen Dank an Quel <qwarlock@gmail.com> fr qRanger, an dessen Tool ich mich orientierte.
--]]

simplerangeindicator_version = 2.3;
simplerangeindicator_on = 1;

local Default = {
	width		= 135,
	height		= 35,
	font = 
	{ textSize	= 12,
      textfont = "Friz Quadrata TT",
	},
	pos	= {},
	friendly = true,
	locked = true
}

SimpleRangeIndicator = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("SimpleRangeIndicator")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil
local RangeCheck = AceLibrary:HasInstance("RangeCheck-1.0") and AceLibrary("RangeCheck-1.0") or nil

function SimpleRangeIndicator:OnInitialize()

	local Options = {
		type = "group",
		args = {
	        lock = {
	            name = "Lock position",
	            type = "toggle",
	            desc = "Lock/Unlock the button.",
	            get = function() return self.db.profile.locked end,
	            set = function( v ) self.db.profile.locked = v end,
	            map = {[false] = "Unlocked", [true] = "Locked"},
				order = 10,
	            guiNameIsMap = true,
            },
			friendly = {
				name = "Show friendly ranges", type = "toggle",
				desc = "Enable/Disable showing friendly target ranges.",
	            get = function() return self.db.profile.friendly end,
	            set = function( v ) self.db.profile.friendly = v self:TargetChanged() end,
	            map = {[false] = "Hostile targets only", [true] = "Hostile and friendly targets"},
	            guiNameIsMap = true,
				order = 20,
			},
			width = {
				name = "Width", type = 'range', min = 10, max = 500, step = 1,
				desc = "Set the width of the button.",
				get = function() return self.db.profile.width end,
				set = function( v )
					self.db.profile.width = v
					self:Layout()
				end,
				order = 30,
			},
			height = {
				name = "Height", type = 'range', min = 5, max = 50, step = 1,
				desc = "Set the height of the button.",
				get = function() return self.db.profile.height end,
				set = function( v )
					self.db.profile.height = v
					self:Layout()
				end,
				order = 40,
			},
			font = {
				name = "Font", type = 'group',
				desc = "Set the font size of different elements.",
				args = {
					text = {
						name = "Font size", type = 'range', min = 6, max = 32, step = 1,
						desc = "Set the font size on the button.",
						get = function() return self.db.profile.font.textSize end,
						set = function( v )
							self.db.profile.font.textSize = v
							self:Layout()
						end
					}
				},
				order = 50,
			},
			background = {
				name = "Background", type = "toggle",
				desc = "Enable/Disable Background.",
	            get = function() return self.db.profile.background end,
	            set = function( v ) self.db.profile.background = v end,
	            map = {[false] = "Text only", [true] = "Text with background"},
	            guiNameIsMap = true,
				order = 60,
			},
		}
	}
	
	if SML then
		Options.args.font.args.textfont = {
			name = "Font face",
			desc = "Typeface to use for the text.",
			type = "text",
			get = function() return self.db.profile.font.textfont end,
			set = function( v )
				self.db.profile.font.textfont = v
				self:Layout()
			end,
			validate = SML:List("font"),
			order = 210
		}
	end

	self:RegisterDB("SimpleRangeIndicatorDB")
	self:RegisterDefaults('profile', Default)
	self:RegisterChatCommand( {"/simplerangeindicator"}, Options )
	self:RegisterChatCommand( {"/sri"}, Options )
	
end

function SimpleRangeIndicator:OnEnable()

	self:RegisterEvent("UNIT_FACTION", "TargetChanged")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
	self:CreateFrameWork()
	
	--	self:ScheduleRepeatingEvent("SimpleRangeIndicator", self.OnUpdate, 0.33, self)
	--	self.master:Show()
end

function SimpleRangeIndicator:SavePosition()

	local x, y = self.master:GetLeft(), self.master:GetTop()
	local s = self.master:GetEffectiveScale()
	
	self.db.profile.pos.x = x * s
	self.db.profile.pos.y = y * s

end

function SimpleRangeIndicator:SetPosition()

	if self.db.profile.pos.x then
		local x = self.db.profile.pos.x
		local y = self.db.profile.pos.y

		local s = self.master:GetEffectiveScale()

		self.master:ClearAllPoints()
		self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	else
		self.master:ClearAllPoints()
		self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end

end

function SimpleRangeIndicator:CreateFrameWork()	

	self.master = CreateFrame("Frame", "SimpleRangeIndicatorFrame", UIParent)
	self.master:Hide()
	
	self.master:SetMovable(true)

	if not self.db.profile.locked then
		self.master:EnableMouse(true)
	else
		self.master:EnableMouse(false)
	end
		
	self.master:RegisterForDrag("LeftButton")
	self.master:SetScript("OnDragStart", function() if not self.db.profile.locked then self["master"]:StartMoving() end end)
	self.master:SetScript("OnDragStop", function() self["master"]:StopMovingOrSizing() self:SavePosition() end)
		
	self.master.Range = self.master:CreateFontString(nil, "ARTWORK")
	self.master.Range:SetFontObject(TextStatusBarText)
    
    self:Layout()
	
end

function SimpleRangeIndicator:Layout()

	local textfont = SML and SML:Fetch("font", self.db.profile.font.textfont) or "Fonts\\FRIZQT__.ttf"

	self.master:SetWidth( self.db.profile.width )
	self.master:SetHeight( self.db.profile.height )

	if(self.db.profile.background) then
		self.master:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
			edgeSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4},
		})
		self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
		self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	end
	
	
	self.master.Range:SetJustifyH("CENTER")
	if(self.db.profile.background) then
		local gameFont, _, oflags = GameFontHighlightSmall:GetFont()
		self.master.Range:SetFont( gameFont, self.db.profile.font.textSize, oflags )
	else
		local gameFont, _, oflags = self.master.Range:GetFont()
		self.master.Range:SetFont( gameFont, self.db.profile.font.textSize, oflags )
		if SML then
		  gameFont = textfont
		end
		self.master.Range:SetFont( gameFont, self.db.profile.font.textSize, "OUTLINE") --[, oflags )--]
	end
	self.master.Range:ClearAllPoints()
	self.master.Range:SetPoint("CENTER", self.master, "CENTER",0,0)
	if(self.db.profile.background) then
		self.master.Range:SetTextColor( 1,1,1 )
	else
		self.master.Range:SetTextColor( TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b )
	end
	
	self:SetPosition()
	
end


--******************************************************************************************************
-- Event handler 

function SimpleRangeIndicator:TargetChanged()
	if UnitExists("target") and not UnitIsDead("target") and (self.db.profile.friendly or UnitCanAttack("player", "target")) then
		index = nil
		self:ScheduleRepeatingEvent("SimpleRangeIndicator", self.OnUpdate, 0.33, self)
		self.master:Show()
	else
		self:CancelScheduledEvent("SimpleRangeIndicator")
		self.master:Hide()
	end
end

function SimpleRangeIndicator:OnUpdate()
	
	local text
	
	if ( (simplerangeindicator_on == 0) or 
	     (nil == UnitName("target") ) ) then
		return;
	end

	if not self.db.profile.locked then
		self.master:EnableMouse(true)
	else
		self.master:EnableMouse(false)
	end

	
	if(UnitExists("target") == 1)then
	
		if(UnitIsUnit("target", "player")) then
			text = format("|c000ffb00 %s: |cffffffffSelf",L["Range"]);
		else
			if(RangeCheck) then
	
				local minrange, maxrange = RangeCheck:getRange("target", false);
				
				if(minrange == nil) then 
					text = format("|cFFF00023 %s: |cffffffffUnknown",L["Range"]);
				else
					if(maxrange == nil) then
						text = format("|cFFF00023 %s: |cffffffff>%d",L["Range"],minrange);
					else
						if(minrange <= 1) then
							text = format("|c000ffb00 %s: |cffffffff%d-%d",L["Range"],minrange,maxrange);
						else
							text = format("|cFFFFD000 %s: |cffffffff%d-%d",L["Range"],minrange,maxrange);
						end
					end
				end
			
			else
				local R1,R2,R3,R4,R5;
				if(CheckInteractDistance("target", 1)) then R1 = 1; else R1 = 0; end
				if(CheckInteractDistance("target", 2)) then R2 = 1; else R2 = 0; end
				if(CheckInteractDistance("target", 3)) then R3 = 1;	else R3 = 0; end
				if(CheckInteractDistance("target", 4)) then R4 = 1; else R4 = 0; end
				if(nil==CheckInteractDistance("target", 4)) then R5 = 1; else R5 = 0; end
				
				if(R4 == 1 and R3 == 0) then
				-- 28m=Y 10m=N
					text = format("|cFFFFD000 %s: |cffffffff10-28",L["Range"]);
				elseif(R3 == 1 and R1 == 0) then
				-- 10m=Y 5m=N
					text = format("|cFFFFD000 %s: |cffffffff5-10",L["Range"]);
				elseif(R1 == 1) then
				-- 5m=Y
					text = format("|c000ffb00 %s: |cffffffff0-5",L["Range"]);
				elseif(R5 == 1) then
				-- 28m=N
					text = format("|cFFF00023 %s: |cffffffff>28",L["Range"]);
				end
			end
		end
	else
		text = " ";
	end

	SimpleRangeIndicator.master:SetBackdropColor( 
		0,
		0,
		0,
		0
	)

	SimpleRangeIndicator.master:SetBackdropBorderColor( 
		0,
		0,
		0,
		0
	)
	
	if( text == " ") then
		SimpleRangeIndicator.master.Range:SetTextColor( 
			0,
			0,
			0,
			0
		)
	end

	SimpleRangeIndicator.master.Range:SetText( text )
end

